class_name Game extends Control

enum Move {
	NONE,
	ROCK,
	PAPER,
	SCISSORS,
}

enum Outcome {
	DEFEAT = -1,
	TIE = 0,
	VICTORY = 1,
}

@export var player_name: String

var player_wins := 0:
	set(value):
		player_wins = value
		player_win_counter.text = str(player_wins)

var opponent_wins := 0:
	set(value):
		opponent_wins = value
		opponent_win_counter.text = str(opponent_wins)

var player_move: Move
var opponent_move: Move

@onready var player_name_label: Label = %PlayerName
@onready var opponent_name: Label = %OpponentName

@onready var player_win_counter: Label = %PlayerWinCounter
@onready var opponent_win_counter: Label = %OpponentWinCounter

@onready var move_select: VBoxContainer = %MoveSelect

@onready var opponent_choosing_move: Label = %OpponentChoosingMove
@onready var opponent_ready: Label = %OpponentReady

@onready var player_move_label: Label = %PlayerMoveLabel
@onready var opponent_move_label: Label = %OpponentMoveLabel
@onready var victory: Label = %Victory
@onready var defeat: Label = %Defeat
@onready var tie: Label = %Tie

@onready var result_animation: AnimationPlayer = %ResultAnimation

@onready var connecting_screen: ColorRect = %ConnectingScreen
@onready var fail_screen: ColorRect = %FailScreen

@onready var default_opp_name := opponent_name.text


func _ready() -> void:
	multiplayer.peer_connected.connect(_on_peer_connected)
	multiplayer.peer_disconnected.connect(_on_peer_disconnected)
	multiplayer.connection_failed.connect(_on_connection_failed)

	player_name_label.text = player_name
	print("Starting client for player ", player_name)


@rpc("any_peer", "call_remote", "reliable", 0)
func set_opponent_name(opp_name: String) -> void:
	opponent_name.text = opp_name


@rpc("any_peer", "call_remote", "reliable", 0)
func set_opponent_move(move: Move) -> void:
	print("Opponent set move: ", move)
	opponent_move = move
	opponent_move_label.text = Move.keys()[opponent_move]
	if move == Move.NONE:
		opponent_ready.hide()
		opponent_choosing_move.show()
	else:
		opponent_choosing_move.hide()
		opponent_ready.show()
	_check_play_condition()


func start_round() -> void:
	move_select.show()
	opponent_ready.hide()
	opponent_choosing_move.show()


func finish_round() -> void:
	match get_outcome():
		Outcome.VICTORY:
			player_wins += 1
		Outcome.DEFEAT:
			opponent_wins += 1
	player_move = Move.NONE
	opponent_move = Move.NONE
	for button: Button in get_tree().get_nodes_in_group("MoveButton"):
		button.set_pressed_no_signal(false)
	start_round()


func _on_peer_connected(id: int) -> void:
	print("PEER CONNECTED ", id)
	set_opponent_name.rpc(player_name)
	opponent_wins = 0
	connecting_screen.hide()
	fail_screen.hide()
	start_round()


func _on_peer_disconnected(id: int) -> void:
	print("PEER DISCONNECTED: ", id)
	set_opponent_name(default_opp_name)
	opponent_wins = 0
	opponent_choosing_move.hide()
	opponent_ready.hide()
	connecting_screen.show()


func _on_connection_failed() -> void:
	print("CONNECTION FAILED!")
	connecting_screen.hide()
	fail_screen.show()


func _set_player_move(value: Move) -> void:
	player_move = value
	player_move_label.text = Move.keys()[player_move]
	set_opponent_move.rpc(player_move)
	_check_play_condition()


func _check_play_condition() -> void:
	if player_move and opponent_move:
		_set_outcome_label(get_outcome())
		move_select.hide()
		opponent_ready.hide()
		opponent_choosing_move.hide()
		result_animation.play("show")


func get_outcome() -> Outcome:
	match player_move:
		Move.ROCK:
			match opponent_move:
				Move.PAPER:
					return Outcome.DEFEAT
				Move.SCISSORS:
					return Outcome.VICTORY
		Move.PAPER:
			match opponent_move:
				Move.SCISSORS:
					return Outcome.DEFEAT
				Move.ROCK:
					return Outcome.VICTORY
		Move.SCISSORS:
			match opponent_move:
				Move.ROCK:
					return Outcome.DEFEAT
				Move.PAPER:
					return Outcome.VICTORY
	return Outcome.TIE


func _set_outcome_label(outcome: Outcome) -> void:
	victory.visible = outcome == Outcome.VICTORY
	defeat.visible = outcome == Outcome.DEFEAT
	tie.visible = outcome == Outcome.TIE


func _on_rock_toggled(toggled_on: bool) -> void:
	_set_player_move(Move.ROCK if toggled_on else Move.NONE)


func _on_paper_toggled(toggled_on: bool) -> void:
	_set_player_move(Move.PAPER if toggled_on else Move.NONE)


func _on_scissors_toggled(toggled_on: bool) -> void:
	_set_player_move(Move.SCISSORS if toggled_on else Move.NONE)
